@@ -23,8 +23,8 @@ _LOGGER = logging.getLogger(__name__)
 
 class RoborockLocalClient(RoborockClient):
 
-    def __init__(self, ip: str, endpoint: str, device_localkey: dict[str, str]):
-        super().__init__(endpoint, device_localkey, True)
+    def __init__(self, ip: str, device_localkey: dict[str, str]):
+        super().__init__("abc", device_localkey)
         self.device_listener: dict[str, RoborockSocketListener] = {
             device_id: RoborockSocketListener(ip, device_id, self.on_message)
             for device_id in device_localkey
@@ -86,8 +86,7 @@ class RoborockSocketListener:
                     await self.on_message(self.device_id, message)
                 except Exception as e:
                     _LOGGER.exception(e)
-            except Exception as e:
-                _LOGGER.exception(e)
+            except BrokenPipeError:
                 self.socket.close()
 
     async def connect(self):
@@ -110,4 +109,6 @@ class RoborockSocketListener:
             raise RoborockTimeout(
                 f"Timeout after {self.timeout} seconds waiting for response"
             ) from None
+        except BrokenPipeError:
+            self.socket.close()
         return response
